"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const install_1 = require("../../install");
const requirements_1 = require("../../requirements");
const ui_1 = require("../../ui");
const Command = require('@netlify/cli-utils');
const { getAddons, createAddon } = require('netlify/src/addons');
const execa = require('execa');
const Listr = require('listr');
class LmSetup extends Command {
    async run() {
        const { flags } = this.parse(LmSetup);
        let helperInstalled = false;
        if (!flags['skip-install']) {
            try {
                helperInstalled = await installHelperIfMissing(flags['force-install']);
            }
            catch (error) {
                this.log(error);
                return;
            }
        }
        const accessToken = await this.authenticate();
        const { api, site } = this.netlify;
        const tasks = new Listr([
            {
                title: 'Provisioning Netlify Large Media',
                task: async function () {
                    await provisionService(accessToken, site.id);
                }
            },
            {
                title: 'Configuring Git LFS for this site',
                task: async function () {
                    await configureLFSURL(site.id, api);
                }
            }
        ]);
        await tasks.run().catch((err) => this.log(err));
        if (helperInstalled) {
            ui_1.printBanner(this, flags['force-install']);
        }
    }
}
LmSetup.description = `Configures your site to use Netlify Large Media.
It runs the install command if you have not installed the dependencies yet.
`;
LmSetup.usage = 'lm:setup';
LmSetup.examples = [
    '$ <%= config.bin %> lm:setup'
];
LmSetup.flags = {
    'skip-install': command_1.flags.boolean({
        char: 's',
        description: 'Skip the credentials helper installation check'
    }),
    'force-install': command_1.flags.boolean({
        char: 'f',
        description: 'Force the credentials helper installation'
    })
};
exports.default = LmSetup;
async function installHelperIfMissing(force) {
    let installHelper = false;
    try {
        const version = await requirements_1.checkHelperVersion();
        if (!version) {
            installHelper = true;
        }
    }
    catch (error) {
        installHelper = true;
    }
    if (installHelper) {
        return install_1.installPlatform(force);
    }
    return false;
}
async function provisionService(accessToken, siteId) {
    if (!siteId) {
        return Promise.reject(new Error('No site id found, please run inside a site folder or `netlify link`'));
    }
    const addons = await getAddons(siteId, accessToken);
    if (typeof addons === 'object' && addons.error) {
        return Promise.reject(addons.error);
    }
    const currentAddon = addons.find((addon) => addon.service_path === '/.netlify/large-media');
    if (currentAddon) {
        return Promise.resolve('Service already provisioned for this site');
    }
    const settings = {
        siteId: siteId,
        addon: 'large-media'
    };
    return createAddon(settings, accessToken);
}
async function configureLFSURL(siteId, api) {
    const siteInfo = await api.getSite({ siteId });
    const url = `https://${siteInfo.id_domain}/.netlify/large-media`;
    return execa('git', ['config', '-f', '.lfsconfig', 'lfs.url', url]);
}
